#!/usr/bin/perl -w

package RPM::Source::SpecSection;

use warnings;
use Carp;
use RPM::Source::Editor;

sub new {
    my $class = shift;
    my $self= {
	TYPE=> 'package',
	PACKAGE=> '',
	BODY=>[],
	@_
    };
    my $TYPE='package';
    my $PACKAGE='';
    if ($self->{BODY}->[0]=~/^\s*\%(\w+)(?:\s+(.+))?\s*$/ && $RPM::Source::Editor::SECTION{$1}) {
	#print "new section: ".$self->{BODY}->[0],"\n";
	$TYPE=$1;
	$PACKAGE=$2;
	$PACKAGE='' unless defined $PACKAGE;
	$PACKAGE=~s/\s*$//;
	$self->{TYPE}=$TYPE;
	$self->{PACKAGE}=$PACKAGE;
    }
    die "Oops! section" if ref $self->{BODY} ne 'ARRAY';
    bless $self, $class;
#    return $self;
}

sub _warn_notapplied {
    my $self=shift;
    print STDERR "@_\n" if $self->{WARN_NOTAPPLIED};
}

sub describe {
    my $self=shift;
    print "SpecSection: type=$self->{TYPE}; package=$self->{PACKAGE}; body: ==>\n".join('',@{$self->{BODY}}),"============== END BODY ====================\n";
}

sub get_type {
    my $self=shift;
    return $self->{TYPE};
}

sub get_package {
    my $self=shift;
    my $pkgname=$self->{PACKAGE};
    $pkgname=~s/\s+-f\s+\S.*//;
    return $pkgname;
}

sub get_package_expanded {
    my $self=shift;
    my $pkg= $self->{PACKAGE};
    return '%{name}' unless $pkg;
    return $pkg if $pkg=~s/-n\s*//;
    return '%{name}-'.$pkg; 
}

sub get_body {
    my $self=shift;
    return $self->{BODY};
}

sub set_body {
    my $self=shift;
    my $ptr=shift;
    die "not an array pointer!" if ref $ptr ne 'ARRAY';
    $self->{BODY}=$ptr;
}

sub unshift_body {
    my $self=shift;
    if ($self->is_head_section()) {
	unshift @{$self->{BODY}}, @_;
    } else {
	my $head = shift @{$self->{BODY}};
	unshift @{$self->{BODY}}, $head, @_;
    }
}

sub unshift_body_before_section {
    my $self=shift;
    unshift @{$self->{BODY}}, @_;
}

sub push_body {
    my $self=shift;
    my $bodyptr=$self->{BODY};
    # move pushed item(s) before %if* block of next section
    my @tail;
    my $i=$#{$bodyptr};
    while ($i>=0 and
	       $bodyptr->[$i]=~/^\s*(?:$|\%if)/ # empty line or condition for the next section 
	) {
	unshift @tail, pop @{$bodyptr};
	$i--;
    }
    push @{$bodyptr}, @_, @tail;
}

sub unshift_body_before {
    my $self=shift;
    my $toinsert = shift;
    my $pattern = shift;
    my $is_applied = 0;
    my @body=();
    my @tail=@{$self->{BODY}};
    # move pushed item(s) before %if* block of next section
    while (@tail>0 and $tail[0]!~/$pattern/ # condition for add
	) {
	push @body, shift @tail;
    }
    if (@tail) {
	push @body, $toinsert, @tail;
	$is_applied = 1;
    }
    $self->{BODY}=\@body;
    $is_applied || $self->_warn_notapplied("not inserted [$toinsert] - pattern [$pattern] not found");
    return $is_applied;
}

sub unshift_body_after {
    my $self=shift;
    my $toinsert = shift;
    my $pattern = shift;
    my $is_applied = 0;
    my @body=();
    my @tail=@{$self->{BODY}};
    while (@tail>0 and $tail[0]!~/$pattern/ # condition for add
	) {
	push @body, shift @tail;
    }
    if (@tail) {
	push @body, shift @tail, $toinsert, @tail;
	$is_applied = 1;
    }
    $self->{BODY}=\@body;
    $is_applied || $self->_warn_notapplied("not inserted [$toinsert] - pattern [$pattern] not found");
    return $is_applied;
}

sub push_body_after {
    my $self=shift;
    my $toinsert = shift;
    my $pattern = shift;
    my $bodyptr=$self->{BODY};
    my $is_applied = 0;
    my @tail;
    my $i=$#{$bodyptr};
    while ($i>=0 and $bodyptr->[$i]!~/$pattern/ # condition for add
	) {
	unshift @tail, pop @{$bodyptr};
	$i--;
    }
    if (@$bodyptr) {
	push @{$bodyptr}, $toinsert, @tail;
	$is_applied = 1;
    }
    $is_applied || $self->_warn_notapplied("not inserted [$toinsert] - pattern [$pattern] not found");
    return $is_applied;
}

sub subst_after {
    my $self=shift;
    my $whatpattern=shift;
    my $replace=shift;
    my $afterpattern = shift;
    my $is_applied = 0;
    my $is_found_after=0;
    my @body=();
    my $count = 0;
    my $maxcount=1;
    my @tail=@{$self->{BODY}};
    while (@tail>0 and $tail[0]!~/$afterpattern/ # condition for add
	) {
	push @body, shift @tail;
    }
    $is_found_after = 1 if @tail;
    while (@tail>0 && $count <= $maxcount) {
	$_=shift @tail;
	#print STDERR "try:$_";
	if (s/$whatpattern/$replace/g) {
	    $is_applied = 1;
	    $count++;
	}
	push @body, $_;
    }
    push @body, @tail;
    $self->set_body(\@body);
    $is_found_after || $self->_warn_notapplied("pattern [$afterpattern] not found");
    $is_applied || $self->_warn_notapplied("not replaced [$whatpattern] after [$afterpattern]");
    return $is_applied;
}


sub empty {
    my $self=shift;
    $#{$self->{BODY}}=0 if ($#{$self->{BODY}}>0);
}

sub delete {
    my $self=shift;
    $self->{BODY}=[];
}

sub is_empty {
    my $self=shift;
    return 1 if $#{$self->{BODY}}<1;
    foreach (@{$self->{BODY}}[1..$#{$self->{BODY}}]){
	#print "is_empty: [$_]" if ! /^\s*(#.*)?$/;
	return 0 if ! /^\s*(#.*)?$/;
    }
    return 1;
}

sub is_head_section {
    my $self=shift;
    return 1 if $self->{TYPE} eq 'package' && $self->{PACKAGE} eq '';
    return 0;
}

sub exclude {
    my $section=shift;
    my $pattern=shift;
    my @out;
    foreach (@{$section->get_body()}) {
	unless (/$pattern/) {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

sub subst {
    my $self=shift;
    my $pattern=shift;
    my $replace=shift;
    Carp::carp "subst: got undefined argument" unless defined $pattern and defined $replace;
    my $is_applied = 0;
    my @out;
    foreach (@{$self->get_body()}) {
	$is_applied++ if s/$pattern/$replace/g;
	push @out, $_;
    }
    $is_applied || $self->_warn_notapplied("not subst [$pattern]");
    $self->set_body(\@out);
    return $is_applied;
}

sub subst_if {
    my $self=shift;
    my $pattern=shift;
    my $replace=shift;
    my $ifpattern=shift;
    Carp::carp 'if_pattern is required' unless $ifpattern;
    my @out;
    my $is_applied = 0;
    foreach (@{$self->get_body()}) {
	if (/$ifpattern/) {
	    $is_applied++ if s/$pattern/$replace/g;
	}
	push @out, $_;
    }
    $is_applied || $self->_warn_notapplied("not subst [$pattern] if [$ifpattern]");
    $self->set_body(\@out);
    return $is_applied;
}

sub replace_line {
    my $section=shift;
    my $stringwhat=shift;
    my $stringreplace=shift;
    my @out;
    foreach (@{$section->get_body()}) {
	$_ = $stringreplace if $_ eq $stringwhat;
	push @out, $_;
    }
    $section->set_body(\@out);
}



sub match {
    my $section=shift;
    my $pattern=shift;
    foreach (@{$section->get_body()}) {
	return 1 if /$pattern/;
    }
    return 0;
}

sub get_tag {
    my $section=shift;
    my $tag=shift;
    my %options = (
	RAW=>0,
	@_);
    foreach (@{$section->get_body()}) {
	if (/^$tag\s*:\s*(\S+)\s*$/) {
	    #warn "tag=$tag raw =$1";
	    my $val=$1;
	    if (!$options{RAW}) {
		&RPM::Source::Editor::macro_subst_inplace($section->{RPM}->{MACRO},\$val);
	    }
	    return $val;
	}
    }
    return;
}

sub set_tag {
    my ($section, $tag, $val)=@_;
    my $specptr=$section->get_body();
    for (my $i=0; $i<=$#{$specptr}; $i++) {
	if ($specptr->[$i]=~/^$tag:(\s*)/) {
	    $specptr->[$i]="$tag:$1$val\n";
	    last;
	}
    }
    return $section;
}

1;
